# Launchpad tools for kubuntu-dev-tools
#
# Copyright (C) 2012 Philip Muskovac <yofel@kubuntu.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from launchpadlib.launchpad import Launchpad
from distutils.version import LooseVersion
import launchpadlib
import os
import stat

class KDTLaunchpad():

    @staticmethod
    def login_with(application_name="kubuntu-dev-tools",
                   service_root=None,
                   launchpadlib_dir=None,
                   timeout=None,
                   proxy_info=None,
                   authorization_engine=None,
                   allow_access_levels=None,
                   max_failed_attempts=None,
                   credentials_file=None,
                   version=Launchpad.DEFAULT_VERSION,
                   consumer_name=None,
                   credential_save_failed=None,
                   credential_store=None):
        ''' Wrapper for Launchpad.login_with() from launchpadlib

        Adds proper credentials file handling and support for various
        environment variables.

        Unless needed, call the function without parameters. If one is set
        the environment variable for that setting is ignored.


        ENVIRONMENT VARIABLES

            KUBUNTU_DEV_TOOLS_CREDENTIALS:
                File where the credentials are stored.

            KUBUNTU_DEV_TOOLS_SERVICE_ROOT:
                Launchpad service that's used. Usually 'staging' or 'production'
                By default production is used.
        '''
        if not credentials_file:
            try:
                credentials_file = os.environ['KUBUNTU_DEV_TOOLS_CREDENTIALS']
            except:
                pass

        if not service_root:
            try:
                service_root = os.environ['KUBUNTU_DEV_TOOLS_SERVICE_ROOT']
            except:
                # default to production
                service_root = "production"

        if credentials_file:
            credentials_file=os.path.expanduser(credentials_file)
            # since launchpadlib will create the file if it doesn't exist make sure
            # the folder it's being created in exists and is user-only rwx
            cred_file_dir = credentials_file[:credentials_file.rfind('/')]
            if not os.path.isdir(cred_file_dir):
                os.makedirs(cred_file_dir)
                os.chmod(cred_file_dir, stat.S_IRWXU)

        # Launchpadlib > 1.9 can't handle consumer_name
        if (LooseVersion(launchpadlib.__version__) > LooseVersion('1.9')):
            return Launchpad.login_with(application_name=application_name,
                    service_root=service_root,
                    launchpadlib_dir=launchpadlib_dir,
                    timeout=timeout,
                    proxy_info=proxy_info,
                    authorization_engine=authorization_engine,
                    allow_access_levels=allow_access_levels,
                    max_failed_attempts=max_failed_attempts,
                    credentials_file=credentials_file,
                    version=version,
                    consumer_name=None, # broken, don't try to use it
                    credential_save_failed=credential_save_failed,
                    credential_store=credential_store)
        else:
            # lucid support (1.6)
            if not allow_access_levels:
                allow_access_levels=[]
            return Launchpad.login_with(consumer_name=application_name,
                   service_root=service_root,
                   launchpadlib_dir=launchpadlib_dir,
                   timeout=timeout,
                   proxy_info=proxy_info,
                   allow_access_levels=allow_access_levels,
                   max_failed_attempts=max_failed_attempts,
                   credentials_file=credentials_file,
                   version=version)
